// Generated by CoffeeScript 2.3.2
(function() {
  // CouchDB back-end
  // ----------------

  // ### Back-end for a regular (static) DB such as provisioning

  // - `db_uri` refers to a database URI.
  // - `view_for` refers to a view object or a function that returns a view object (based on a requested key). A view object contains `view.map` (the map function as a function), `view.app`,and `view.name`, which all refer to the same view. If `view_for` is `null`, the `_all_docs` "identity" "view" is used.

  // Changes are not computed for views where the `view_for` parameter is a function. (See below for the reason.) This could be worked-around by providing a database-to-view mapper that records the outcome of the view in a new database (and we are asked to monitor that database). (But notice that regular CouchDB databases are not suitable for this, since they can't directly handle non-string ids, and more importantly, cannot deal with duplicated keys/ids either.)
  var CouchDB, build_backend, changes_view, couchdb_backend, most, semantic_for, view_as_stream;

  couchdb_backend = function(db_uri, view_for, fromJS, use_lru, limiter) {
    var changes, get_key, semantic, the_db, view_changes, view_key;
    the_db = new CouchDB(db_uri, use_lru);
    semantic = semantic_for(the_db, fromJS);
    get_key = function(key) {
      return the_db.get(key).catch(function() {
        return null;
      });
    };
    // Document changes
    changes = the_db.changes({
      include_docs: true
    }).multicast();
    // Changeset for wandering-country-view/all (or other view)
    view_changes = (function() {
      switch ((view_for != null) && typeof view_for) {
        // For a static `view_for` object, we apply the `.map` function locally to each modified document,
        // therefor dynamically computing the outcome of the view. (CouchDB does the same thing on its side,
        // so if we need to reconnect we can query the CouchDB view and obtain the same results.)
        case 'object':
          return changes_view(view_for.map, changes).multicast();
        default:
          return most.empty();
      }
    })();
    // We do not provide support for view changes for `view_for` functions.

    // The reason this is a bad idea is that the view to use is decided based on the keys
    // (see below for `.app` and `.name`), but the keys are _generated_ by the very
    // same `map` function we're trying to select. (If this wasn't the case, the results would be
    // inconsistent between the data returned by CouchDB, and the data generated by the function.)
    // In details: to use this feature, you would have to provide a function would have to know
    // how to reliably predict which keys will be returned by which ids (in other words it would
    // be able to pre-compute the outcome of the `map` function).

    // Also, we do not provide support for view changes for aggregate views (`count`, `range` etc.).

    // On initial subscription we convert the key into a stream, using the server-side (CouchDB-stored) view.
    view_key = (function() {
      switch ((view_for != null) && typeof view_for) {
        case false:
          return function(key) {
            return view_as_stream(the_db, null, '_all_docs', key, limiter);
          };
        case 'object': // there is only one DB, or the view is named the same in all DBs
          return function(key) {
            return view_as_stream(the_db, view_for.app, view_for.name, key, limiter);
          };
        case 'function':
          return function(key) {
            var view;
            view = view_for(key);
            if (view == null) {
              return most.emtpy();
            }
            return view_as_stream(the_db, view.app, view.name, key, limiter);
          };
        default:
          throw new Error('view_for must be object or function');
      }
    })();
    return build_backend({changes, view_changes, semantic, get_key, view_key, fromJS});
  };

  module.exports = couchdb_backend;

  changes_view = require('most-couchdb/changes-view');

  view_as_stream = require('./util/view');

  most = require('most');

  CouchDB = require('most-couchdb');

  semantic_for = require('./util/semantic-for');

  build_backend = require('./util/build-backend');

  // FIXME: figure out the pattern to push in bulk

// The semantic really is 'UPDATE', not 'OVERWRITE'
//   → we could either do bulk-get / bulk-put (aka `_all_docs` and `_bulk_docs`),
//   → or use an [update function](http://docs.couchdb.org/en/2.1.1/api/ddoc/render.html#db-design-design-doc-update-update-name)
// For now the code (above) does single GET/PUT on updates.

}).call(this);
